// Bubble Game by Johannes Wallroth, Germany
// Visit "www.programming.de" for more C# examples!

using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace BubbleGame
{

class MainForm : Form
{
	Matrix matrix = new Matrix();
	StatusBar statusBar = new StatusBar();
	StatusBarPanel sbPnl1 = new StatusBarPanel();
	StatusBarPanel sbPnl2 = new StatusBarPanel();
	MainMenu menu;		
	Timer timer = new Timer();		
	int clicks, ticker;
		
	MainForm()
	{
		Text = "Bubble Game";
		ClientSize = new Size(321, 362);
		MaximizeBox = false;		
		FormBorderStyle = FormBorderStyle.FixedSingle;		
		
		MouseDown += new MouseEventHandler(OnMouseDown);	
		MouseMove += new MouseEventHandler(OnMouseMove);	
		MouseLeave += new EventHandler(OnMouseLeave);	
				
		CenterToScreen();
				
		Paint += new PaintEventHandler(OnPaint);
		
		SetStyle(ControlStyles.UserPaint, true);
		SetStyle(ControlStyles.AllPaintingInWmPaint, true);
		SetStyle(ControlStyles.DoubleBuffer, true);
		
		timer.Interval = 15;	
		timer.Tick += new EventHandler(Ticker);
		timer.Start();
		
		menu = new MainMenu();
		MenuItem miProgram = menu.MenuItems.Add("&Program");
		miProgram.MenuItems.Add(new MenuItem("&New Game", new EventHandler(New_Clicked), Shortcut.CtrlN));		
		miProgram.MenuItems.Add(new MenuItem("&Info about...", new EventHandler(About_Clicked)));		
		miProgram.MenuItems.Add(new MenuItem("&Exit", new EventHandler(Exit_Clicked)));
		
		this.Menu = menu;
		
		statusBar.ShowPanels = true;		
		statusBar.SizingGrip = false;				
		statusBar.Panels.Add(sbPnl1);		
		statusBar.Panels.Add(sbPnl2);		
		sbPnl1.BorderStyle = sbPnl2.BorderStyle = StatusBarPanelBorderStyle.None;	
		sbPnl1.Width = 250;		
		sbPnl2.AutoSize = StatusBarPanelAutoSize.Spring;		
		Controls.Add(statusBar);		
	}
	
	void OnMouseDown(object sender, MouseEventArgs e)
	{	
		if (e.Button == MouseButtons.Left && ticker > 10)
        {
        	ticker = 0;
        	matrix.rearrange = false;
			clicks ++;
			matrix.SwitchTiles();        	
        }       
	}
	
	void OnMouseMove(object sender, MouseEventArgs e)
	{		
		Point p = new Point(e.X, e.Y);
		matrix.MarkTiles(p);
	}
	
	void OnMouseLeave(object sender, EventArgs e)
	{
		matrix.UnmarkTiles();	
	}
	
	void Ticker(object sender, EventArgs e)
	{				
		ticker ++;
		
		if(clicks == 0)
			sbPnl1.Text = "Remove all Bubbles by making Couples!";
		else
			sbPnl1.Text = matrix.TileNumber() + " Bubbles left";
		
		sbPnl2.Text = clicks + " Click" + (clicks == 1 ? "" : "s");
		
		Invalidate();
	}
	
	void OnPaint(Object sender, PaintEventArgs e)  
	{		
	   	Graphics g = e.Graphics ; 				
		g.SmoothingMode = SmoothingMode.AntiAlias;
		
		DrawBackground(g);		
		matrix.Draw(g);	
	} 
	
	void DrawBackground(Graphics g)
	{	
		LinearGradientBrush gradientBrush = 
			new LinearGradientBrush(ClientRectangle, Color.DarkBlue, Color.Black, LinearGradientMode.Vertical); 
		
	  	g.FillRectangle(gradientBrush, ClientRectangle); 
		
		Pen pen = new Pen( Color.PowderBlue, 2 ); 
		
		for(int i = 0; i < ClientSize.Width * 2; i += 20)
			g.DrawLine( pen, 0, i, i, 0 ); 
	}
	
	void New_Clicked(object sender, EventArgs e)
	{
		clicks = 0;
		matrix = new Matrix();	
	}
	
	void About_Clicked(object sender, EventArgs e)
	{
		MessageBox.Show("C# Example by Johannes Wallroth\nwww.programming.de", "Bubble Game", 
		                MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
	}
	
	void Exit_Clicked(object sender, EventArgs e)
	{
		this.Close();
	}
		
	static void Main(string[] args)
	{
		Application.Run(new MainForm());
	}
}
}
